function [p0, eCov, ev, evar0] = black_box( l, QQ, p0, ...
    g_fun, c_fun, h_fun, update_fun, varargin )

% general adjustment procedure.
%
% [p, eCov_pp, ev, evar0] = black_box( l, Cov_ll, p0, ...
%         g_fun, c_fun, h_fun, update_fun, ... )
%
% estimates the parameter p because of the implicit constrants g(l,p)=0
% for the observations l and the parameters p
% with the parameter constraints h(p)=0
% and the observations constraints c(l)=0.
%
% input:
%
%      l: [N,1] vector of N observations
% Cov_ll: [N,N] covariance matrix of the observations l
%     p0: [U,1] vector of approximate values for the U parameters p
%
% g_fun: Name of the problem-specific function [g, AA, BB] = g(l, p)
%        which returns the G constradictions g because of g(l,p)=0,
%        its [G,U] Jacobian AA  w.r.t. the parameters p, and
%        the [G,N] Jacobian BB  w.r.t. the observations l.
%
% c_fun: Name of the problem-specific function [c, CC] = c(l) which
%        returns the C contradictions c because of c(l)=0 and
%        its [C,N] Jacobian CC  w.r.t the obervations l.
%
% h_fun: Name of the problem-specific function [h, HH] = h(p) which
%       returns the H contradictions h because of h(p)=0 and
%       its [H,U] Jacobian HH  w.r.t the parmeters p.
%
% update_fun: Name of the problem-specific function  Cov_ll = f(l0, Cov_ll)
%             which  returns the updated covariance matrix Cov_ll 
%             of the observations and its inverse.
%
% The function names can be specified via
%       black_box( l, Cov_ll, p0,   @my_gfun, @my_hfun, @my_cfun, ...
%              @myupdate_fun, ...);
%
% Additional parameters, e.g. index vectors or constants, are passed
% identically to the three problem-specific functions gfun, hfun, and cfun.
%
% Please note that the data normalization (conditioning) is up to the user.
%
%
% output:
%
%       p: [U,1] vector of estimated parameters
% eCov_pp: [U,U] estimated covariance matrix of the estimated parameters
%      ev: [N,1] corrections for the parameters (negative residuals)
%                Thus the adjusted observations are:  l+ev.
%   evar0: [1,1] estimated variance factor
%
% For the adjustment model and its applications see
%
% [1] Jochen Meidow, Christian Beder and Wolfgang Frstner (2009)
%     "Reasoning with Uncertain Points, Straight Lines, and Straight Line
%     Segments in 2D". ISPRS Journal of Photogrammetry and Remote Sensing, 
%     vol. 64, no. 2, p. 125-139
% [2] Jochen Meidow, Wolfgang Frstner and Christian Beder (2009)
%     "Optimal Parameter Estimation with Homogeneous Entities and Arbitrary
%     Constraints". In: Pattern Recognition, Proceedings of the 31st DAGM 
%     Symposium, Jena, Germany, p. 292-301, Springer
%
% author:   J. Meidow, Fraunhofer-FOM, Ettlingen
%



%% procedure parameters
maxIter = 5;
verbose = true;

%% initialization
U = length(p0);   % number of parameters
N = length(l);    % number of obervations

lo = l;           % approximation values for adjusted observations


%% iterations
for it = 1:maxIter
    if verbose
        fprintf('iteration %2d/%d\t', it, maxIter)
    end

    %% Jacobians & contradictions
    [cg, AA, BB] = g_fun( lo, p0, varargin{:} );   % constraints  g(l,p) = 0
    [cl, CC]     = c_fun( lo,     varargin{:} );   % constraints  c(l  ) = 0
    [ch, HH]     = h_fun(     p0, varargin{:} );   % constraints  h(  p) = 0

    % corresponding numbers
    G = length( cg );
    H = length( ch );
    K = length( cl );

    % auxillary variable
    a = BB*CC'* inv(CC*CC')* ( CC*(l-lo)  +cl)  -BB*(l-lo) -cg;

    % inverse covariance matrix of contradictions g
    iCov_gg = inv( BB*QQ*BB');

    % reduced normal equation system
    NN = [ ...
        AA'*iCov_gg*AA,  HH'; ...
        HH,      zeros(H,H)....
        ];
    tt = [...
        AA'*iCov_gg*a; ...
        -ch ...
        ];

    % solve for [edp; mu]  via LU decomposition
    [LLL,UUU] = lu(NN);
    edp = UUU\(LLL\tt);
    edp = edp(1:U);                   % drop Lagrangian factor\mu

    % Langrangian factor \lambda
    lambda = iCov_gg*( AA*edp - a );

    % estimated corrections (negative residuals)
    ev = -QQ*BB'*lambda -CC'* inv(CC*CC')* ( CC*(l-lo) +cl);

    % sum of squared and weighted residuals
    omega = lambda'*( AA*edp - a );   % i.e.   omega = ev'*pinv(QQ)*ev;

    p0 = p0 +edp;                     % update parameters
    lo = l  +ev;                      % update observations
    QQ = update_fun( lo, QQ );        % update cov. matrix

    if verbose
        fprintf(' ||delta|| = %12g\t', norm( edp) )
        fprintf('omega = %g\n', omega)
    end
    
end % loop iterations

r = G -U +H;                % redundancy
evar0 = omega/r;            % estimated variance factor

%% estimated covariance matrix of the estimated parameters
eCov = evar0*inv(NN);
eCov = eCov(1:U,1:U);

%% summary
if verbose
    fprintf('\n');
    fprintf('     parameters   p:   %6d\n', U)
    fprintf('   observations   l:   %6d\n', N)

    fprintf(' constraints g(l,p):   %6d\n', G)
    fprintf(' constraints   h(p):   %6d\n', H)
    fprintf(' constraints   c(l):   %6d\n', K)

    fprintf('         redundancy:   %6d\n', r)
    fprintf('       relative redundancy:       %.2g\n\n', r/G)
    fprintf('estimated condition number N:     %g\n', condest(NN) )
    fprintf('estimated condition number B''QB:  %g\n', condest(BB*QQ*BB') )
    fprintf('estimated variance factor:        %g\n\n', evar0 );

    el = l+ev;  % adjusted observations
    fprintf('\tcheck g(l+^v,^p) = %+g\n', gfun(el, p0, varargin{:}))
    fprintf('\tcheck      h(^p) = %+g\n', hfun(    p0, varargin{:}))
    fprintf('\tcheck c(l+^v)    = %+g\n', cfun(el,     varargin{:}))
    
end

return