function demo_optim_estim_F_matrix(R)

% estimation of the fundamental matrix in homogeneous coordinates.
%
% DEMO_OPTIM_ESTIM_F_MATRIX demonstrates the estimation of the fundamental
% matrix F presented at the DAGM symposium 2009 in Jena. The function 
% performs 100 Monte Carlo simulations and evaluates the corresponding
% results by computing the Mahalanobis distances and their empirical
% distribution.
%
% The constraints for the Matrix F and the point pairs (x1, x2)_i are
% (1)      x1i'*F*x2i = 0
% (2)          det(F) = 0
%            f'*f - 1 = 0    with  f = vec(F)
% (3)    x1i'*x1i - 1 = 0
%        x1i'*x1i - 1 = 0,
%
% DEMO_OPTIM_ESTIM_F_MATRIX(R) performs R simulations.
%
% author: J. Meidow, Fraunhofer-FOM, Ettlingen


%% (1) generation synthetic data
n3dPts = 30;                % number of 3d points, normal distributed
sigma_n = 0.005;            % noise for image points in approx. [-1,1]
display( sigma_n )
if nargin==0
    R = 100;                % number of simulations to run
end

% 1st camera:
K1 =  [ ...                 % calibration matrix
    2 0 0; ...
    0 2 0; ...
    0 0 1 ...
    ];
R1 = eye(3);                % rotation matrix
Z1 = [0 0 -6]';             % projection centre
P1 = K1*R1*[eye(3), -Z1];   % projection matrix
K2 = K1;                    % calibraton matrix 2nd camera
  
%% parameters goodness-of-fit test
ks.hypothesis = 'unequal';
ks.alpha = 0.05;

%% memory allocation 
Mahalanobis = zeros(R,1);   % Mahalanobis distances
evar0       = zeros(R,1);   % estimated variance factors \sigma_0^2

%% R simulation runs
wb = waitbar(0, 'Please wait...');
for run=1:R
    
    waitbar( run/R, wb);
    
    % 2nd camera:               
    r = [1*randn 1*randn   1+0.1*randn]';  % random rotation
    r = 6*r/norm(r);
    R2 = Rot_v1v2( [0 0 1], r)';           % rotation matrix
    Z2 = -r;                               % projection centre
    P2 = K2*R2*[eye(3), -Z2];              % projection matrix

    % generate 3d points, true fundamental matrix, and noisy observations
    [x1, x2, Fo, F_true, X] = gen_synth_data(P1, P2, n3dPts, sigma_n );

    % plot scene
    if 0
        figure(3)
        %clf
        plot_camera(P1, 0.5,1);
        plot_camera(P2, 0.5,1);
        hold on
        plot3(X(1,:), X(2,:), X(3,:), 'k.', 'MarkerSize', 12)
        plot3([-4 4],[ 0 0], [0 0], 'k-')
        plot3([ 0, 0],[-4 4],[0 0], 'k-')
        plot3([ 0, 0],[ 0,0],[-4 4], 'k-')
        axis equal tight
        grid on
        box on
        pause
    end
    
    % spherical normalization, stack vector l and block-diagonal matrix Q
    N = numel(x1);
    nG =   size(x1,2);
    Cov = 10^2*diag(  [1 1 0].^2  );
    QQ = spalloc( N, N, 3*N) ;
    l = zeros(N,1);

    for i=1:nG

        idx1 = 6*(i-1)+(1:3);
        idx2 = 6*(i-1)+(4:6);

        [ l(idx1), QQ(idx1,idx1)] = spherical_normalization( x1(:,i), Cov);
        [ l(idx2), QQ(idx2,idx2)] = spherical_normalization( x2(:,i), Cov);

    end

    %% parameter estimation
    p = Fo(:);            %   p'*p = 1   holds for approximate values p
    [p, eCov, ev, evar0(run)] = black_box( ...
        l, QQ, p, ...
        @gfun, @cfun, @hfun, ...
        @update_cov );
    % fprintf('estimated variance factor = %g\n', evar0(run) );
 

    f = F_true(:);           % true parameters vec(F)
    f = f/sign( f(1));
    
    ef = p;                  % estimated parameters vec(^F)
    ef = ef/sign(ef(1));
    
    % Mahalanobis distances
    Mahalanobis(run) = (ef-f)'*pinv(eCov)*(ef-f);

end % loop runs

close(wb)                    % close waitbar figure


%% empirical distribution \chi^2_7
figure(1); clf;


dof = 7;  display(dof)
x = 0:1:30;
H = hist( Mahalanobis, x);
h(1) = bar(x,H/sum(H)/(x(2)-x(1)), ...
    'BarWidth', 1.0, ...
    'FaceColor', 0.5*[1 1 1] ...
    );
axis tight, hold on
y = chi2pdf(x,dof);
h(2) = plot(x, y, 'k-', 'LineWidth', 3);
axis tight
legend(h, sprintf('%d samples', length(Mahalanobis) ), '\chi^2_7')
title('Distribution of Mahalanobis distance empirical/theoretical');
hold off

% Single sample Kolmogorov-Smirnov goodness-of-fit hypothesis test

x = linspace( min(Mahalanobis), max(Mahalanobis), 300);
y = chi2cdf(x,dof);
H_KS = kstest( Mahalanobis, [x(:),y(:)], ks.alpha, ks.hypothesis);
display(H_KS)

%% variance factor
figure(2); clf;

dof = n3dPts-7;    display(dof)
T = dof* evar0/(sigma_n^2)*100;   % test statistic
% T = evar0/(sigma_n^2)*100;   % ~ 1
[H,x] = hist( T, 30);  %
h(1) = bar( x, H/sum(H)/(x(2)-x(1)), ...
    'BarWidth', 1.0, ...
    'FaceColor', 0.5*[1 1 1], ...
    'LineWidth',0.5 ...
    );
axis tight
hold on

y = chi2pdf(x, dof);
h(2) = plot(x,y,'k-', 'linewidth',3);
axis tight
legend( h, sprintf('%d samples', length(evar0)), ...
           ['\chi^2_{', num2str(dof), '}'] );
title('Distribution of variance factor empirical/theoretical');       

% Single sample Kolmogorov-Smirnov goodness-of-fit hypothesis test
x = linspace( min(evar0), max(evar0), 300);
y = chi2cdf(x,dof);
H_KS = kstest( evar0, [x(:),y(:)], ks.alpha, ks.hypothesis);
display(H_KS)


disp('finished.')

return