function demo_optim_estim_vanishing_points

% The vanishing points example for the general adjustment model.
%
% DEMO_OPTIM_ESTIM_VANISHING_POINTS demonstrates the constrained 
% vanishing points determination in homogneous coordinates presented 
% at the DAGM symposium 2009 in Jena. The example shows the first 
% image of the 'corridor sequence' provided by the Visual Geometry 
% Group (VGG), University of Oxford.
%
% The function loads the image and the straight line segments from 
% the file <prepared.mat> and performs the corresponding least
% squares adjustment with 3 vanishing points with the 3 othogonality
% constraints
%        v1'*IAC*v2 = 0,
%        v1'*IAC*v3 = 0,
% and    v2'*IAC*v3 = 0
% with the image of the absolute conic IAC.
%
% author: J. Meidow, Fraunhofer-FOM, Ettlingen


%% load image, calibration matrix, and extrated image features
load 'prepared.mat' K I vanish

%% plot image, feature extraction, and classification
imshow(I)   % display image
hold on

% plot segments a-b
color = 'rgb';
for v=1:3
    vanish(v)
    for s=1:size( vanish(v).a,2)
        plot( [vanish(v).a(1,s), vanish(v).b(1,s)], [vanish(v).a(2,s), vanish(v).b(2,s)], [color(v) '-']);
    end
end
figure(gcf)



%% scale data and parameters
Sp = [ ...
    1/500  0 -300/500; ...
    0  1/500 -300/500; ...
    0  0 1];

[K, vanish] = scale(Sp, K, vanish);

IAC = inv(K*K');      %  image of the absolute conic, ~I


%% observations: stack l & Q

N = 3*( size(vanish(1).l,2) +size(vanish(2).l,2) +size(vanish(3).l,2) );
l  = zeros(N,1);
QQ = spalloc( N, N, 3*N) ;


% stack l and Qll
il = 1:3;
idx = [];
for v=1:3
    for i=1:size(vanish(v).l,2)

        l(il)      = vanish(v).l(:,i);
        QQ(il,il) = vanish(v).Cov(:,:,i);
        il = il +3;

        idx(end+1)= v;
    end
end

p0 = [ ...
    vanish(1).pt; ...
    vanish(2).pt; ...
    vanish(3).pt;
    ];
display( p0 )

hold on
v0 = p0(4:6);
v0 = inv(Sp)*v0;
h = plot( v0(1)/v0(3), v0(2)/v0(3), ['o' color(2)] );  % approximate value
legend(h, 'approximate values')
plothandle_approximate=h;

%% estimation
[p, eCov] = black_box( ...
    l, QQ, p0, ...
    @gfun, @cfun, @hfun, ...
    @update_cov, ...
    idx, IAC);


%% rescale with error propagation
[K, p, eCov] = rescale(Sp, K, p, eCov);

disp('estimated vanishing points (three homogeneous vectors):') , disp(p)
disp('estimated std dev and correlations'),  disp( SigmaRhoMat(eCov) );


%% plot sphere
k = 10;
figure(3)
clf
hold on
set(gcf,'name','spherical representation')
set(gcf,'NumberTitle','off')
plot_gnomonic_projection( K, I,    p, eCov,   k^2, color)
hold off
axis tight
view(55,40)

%% plot 2nd vanishing point
figure(1)
idx = 4:6; % 2nd
v = p(idx);
Cov_vv = eCov(idx,idx);
[v, Cov_vv] = Euclidean_normalization( v, Cov_vv, 'straight line');
Con_vv = adj(k^2*Cov_vv - v*v');
hold on
h = plot_conic_explicit(Con_vv, 'color', color(2) );
set(h(1), 'Marker', '+')
legend([plothandle_approximate h(1)], 'approximate values','estimation')

disp('finished.')

return


%% scale parameters and data
function [K, vanish] = scale(Sp, K, vanish)

Sl = inv(Sp)';

K = Sp*K;             %  K ~= I  now

for v=1:3
    vanish(v).pt = Sp*vanish(v).pt;
    vanish(v).pt = vanish(v).pt / norm( vanish(v).pt);

    for l=1:size( vanish(v).l,2)
        [ vanish(v).l(:,l), vanish(v).Cov(:,:,l) ] = ...
            spherical_normalization( Sl*vanish(v).l(:,l),  Sl*vanish(v).Cov(:,:,l)*Sl');
    end
end

return

%% rescale parameters with error propagation
function [K, p, eCov] = rescale(Sp, K, p, eCov)

K = inv(Sp)*K;
J = blkdiag( inv(Sp), inv(Sp), inv(Sp) );
p = J*p;
eCov = J *eCov* J';

return
