function demo_optim_estim_hip_roof

% The hip roof example for the general adjustment model.
%
% DEMO_OPTIM_ESTIM_HIP_ROOF demonstrates the geometric reasoning in 
% homogeneous coordinates for the 'hip roof' example presented at the 
% DAGM symposium 2009 in Jena. The function loads the image and the
% extracted image features (points and straight line segments with 
% covariance matrics) in the  file <prepared.mat> and performs the 
% corresponding least squares adjustment for the 6 corners of the hip 
% roof.
%
% author: J. Meidow, Fraunhofer-FOM, Ettlingen


load prepared

k = 10;    % enlargement of uncertainty regions

%% cconditioning
mu = [ 150; 150];
% plot( mu(1), mu(2), 'x')
% 
H = [...   
    1  0  -mu(1); ...
    0  1  -mu(2); ...
    0  0      50
    ];

%%  show image
clf
imshow(I)
figure(gcf)
hold on

%% plot observations
E = length( e );

for j=1:E

    % disp(e{j})
    switch e{j}.type
        case '2D point'

            % plot
            Cov = e{j}.Cov;
            v   = e{j}.v;
            [v,Cov] = Euclidean_normalization(v, Cov, 'point');
            Con = adj( k^2*Cov -v*v');              % point conic
            h = plot_conic_explicit( Con,  'Color', 'b', 'LineWidth', 4);
            h = plot_conic_explicit( Con,  'Color', 'c', 'LineWidth', 1);
            text( v(1)/v(3)+2, v(2)/v(3)+2, num2str(j), 'Color', 'r', ...
                'FontSize', 14);
            set( h(1), 'Color', 'b', 'MarkerSize', 20);
 
            
        case '2D straight line segment'
            
            % plot end points
            Cov_aa = e{j}.Cov_aa;       Cov_aa(3,3) = 0;
            a = e{j}.a(:);              a(3) = 1;
            Con_aa = adj( k^2*Cov_aa -a*a');
            plot_conic_explicit( Con_aa, 'LineStyle', '-', 'Color', 'c');
            
            
            Cov_bb = e{j}.Cov_bb;       Cov_bb(3,3) = 0;
            b = e{j}.b(:);              b(3) = 1;
            Con_bb = adj( k^2*Cov_bb -b*b');
            plot_conic_explicit( Con_bb, 'LineStyle', '-', 'Color', 'c');
            
            % plot segment
            plot( [a(1) b(1)], [a(2) b(2)], 'b-', 'LineWidth', 4);
            text( (a(1)+b(1) )/2+2, (a(2) +b(2))/2+2, num2str(j), ...
                'Color', 'r', 'FontSize', 14)


            % straight line
            l = e{j}.v;
            Cov_ll = e{j}.Cov;
            [l,Cov_ll] = Euclidean_normalization(l, Cov_ll, 'straight line');
            Con = l*l' -k^2*Cov_ll;  % line conic, Hartley & Z., (11.15), p. 300
            plot_conic_explicit( Con, 'LineStyle', '-', 'Color', 'b', 'LineWidth', 5);
            plot_conic_explicit( Con, 'LineStyle', '-', 'Color', 'c', 'LineWidth', 1);
         
            
        otherwise
            error('unknown entity <%s>', e{j}.type);


    end
    figure(gcf)
    pause(0.1)
end


figure(gcf)


%% observations: conditioning & spherical normalization
iH = inv(H);
for j=1:E
    
    switch e{j}.type
        case '2D point'
            e{j}.v   = H*e{j}.v;
            e{j}.Cov = H*e{j}.Cov*H';
            
        case '2D straight line segment'
            e{j}.v   = iH'*e{j}.v;
            e{j}.Cov = iH'*e{j}.Cov*iH;
            
        otherwise
            error( e{j}.type )
    end
    
    [e{j}.v, e{j}.Cov] = spherical_normalization( e{j}.v, e{j}.Cov);
end



%% define geometric bi-relations
GGG = sparse( E, 6);

% 1. indentities
%  point/point
GGG( 1, 1) = 2;
GGG( 2, 2) = 2;
GGG( 3, 3) = 2;
GGG( 4, 4) = 2;
GGG( 5, 5) = 2;

% 2. incidences
% % line/point
nPts = 5;
GGG( 1+nPts ,1) = 1;
GGG( 2+nPts, 1) = 1;
GGG( 5+nPts, 1) = 1;
GGG( 7+nPts, 1) = 1;

GGG( 4+nPts, 2) = 1;
GGG( 5+nPts, 2) = 1;
GGG( 8+nPts, 2) = 1;

GGG( 4+nPts, 3) = 1;
GGG( 6+nPts, 3) = 1;

GGG( 3+nPts, 4) = 1;
GGG( 9+nPts, 4) = 1;

GGG( 3+nPts, 5) = 1;

GGG( 7+nPts, 5) = 1;
GGG( 8+nPts, 5) = 1;

GGG( 1+nPts, 6) = 1;
GGG( 2+nPts, 6) = 1;
GGG( 6+nPts, 6) = 1;
GGG( 9+nPts, 6) = 1;


% contraints observ
CCC = sparse( E,E);
for i=1:E
    CCC(i,i) = 1;  % normalization
end



%% parameters
for i=1:5
    p{i} = e{i};
end

% parameter restrictions
HHH = sparse( 6, 6);
for i=1:6
    HHH(i,i) = 1; % normaliztation
end


%% parameters
p0 = [];
for i=1:5
    x = e{i}.v;
    x = x/norm(x);
    p0 = [p0; x ];
end

% point #6
x6 = iH*cross( e{6}.v, e{11}.v );
plot( x6(1)/x6(3), x6(2)/x6(3), 'c+');
x6 =  H*x6;          % conditioning
x6 = x6/norm(x6);
p0 = [p0; x6];


%% estimation

% stack
l = [];
Cov_ll = [];
for i=1:length(e)
    l = [l; e{i}.v];
    Cov_ll = blkdiag( Cov_ll, e{i}.Cov);
end

[p, eCov_pp, ev, evar0] = black_box( l, Cov_ll, p0, ...
    @gfun, @cfun, @hfun, @update_cov, ...
    GGG, CCC, HHH );




%% plot results
hold on
for i=1:length(p)/3
    idx = 3*(i-1)+(1:3);

    % plot points
    v    =  p(idx);
    Cov  = eCov_pp(idx,idx)/evar0;
    
    [v,Cov] = Euclidean_normalization( v, Cov, 'point');
    Con  = adj( k^2*Cov -v*v');
    plot_conic_explicit ( H'*Con*H, 'Color' ,'r', 'LineWidth', 5);
    plot_conic_explicit ( H'*Con*H, 'Color' ,'m', 'LineWidth', 1);
    v = iH*v;
    r{i}.x = v(1)/v(3);
    r{i}.y = v(2)/v(3);
    text( v(1)/v(3)-4, v(2)/v(3)-4, num2str(i), ...
        'FontSize', 14, ...
        'Color','r');

end


%% outline of hip roof
S = zeros(6,6);
S(1,2) = 1;
S(1,5) = 1;
S(1,6) = 1;

S(2,5) = 1;
S(2,3) = 1;

S(3,4) = 1;
S(3,6) = 1;
S(4,5) = 1;
S(4,6) = 1;
S = S+S';
for i=1:size(S,1)
    for j=i:size(S,2)
        if S(i,j) ~= 0
            plot( [r{i}.x r{j}.x], [r{i}.y, r{j}.y] , 'y-', ...
                'LineWidth', 2)
        end
    end
end

% legend(h(2), 'estim. param.')

fprintf('\nfinished.\n')

return